/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */;
open MatcherTypes;

type intMatchers = {toBe: int => unit};
type intMatchersWithNot = {
  not: intMatchers,
  toBe: int => unit,
};

let passMessageThunk = () => "";

let makeMatchers = (accessorPath, {createMatcher}) => {
  let createIntMatchers = actual => {
    let toBe = isNot =>
      createMatcher(
        (
          {matcherHint, formatReceived, formatExpected},
          actualThunk,
          expectedThunk,
        ) => {
        let actual = actualThunk();
        let expected = expectedThunk();
        let actualEqualsExpected = actual == expected;
        let pass =
          actualEqualsExpected && !isNot || !actualEqualsExpected && isNot;
        if (!pass) {
          let message =
            String.concat(
              "",
              [
                matcherHint(
                  ~expectType=accessorPath,
                  ~matcherName=".toBe",
                  ~isNot,
                  (),
                ),
                "\n\n",
                "Expected: ",
                formatExpected(string_of_int(expected)),
                "\n",
                "Received: ",
                formatReceived(string_of_int(actual)),
              ],
            );
          (() => message, pass);
        } else {
          (passMessageThunk, pass);
        };
      });

    let makeIntMatchers = isNot => {
      toBe: expected => toBe(isNot, () => actual, () => expected),
    };

    let intMatchers = makeIntMatchers(false);
    {not: makeIntMatchers(true), toBe: intMatchers.toBe};
  };
  createIntMatchers;
};
